/*
 * diskinf.c - show disk information
 * original code:SASANO Takayoshi
 * --- public domain, no warranty.
 */

#include <basic.h>
#include <bstdio.h>
#include <bstdlib.h>
#include <errcode.h>
#include <btron/btron.h>
#include <device/disk.h>

#ifdef DEBUG
#define	DP(x)	printf x
#else
#define	DP(x)	/* */
#endif

EXPORT	WERR	main(W argc, TC *argv[])
{
	WERR	err = ER_PAR;
	W	i, dd, rsz;
	ID	id;
	DiskInfo	diskinfo;
	VP	vp;
	DiskPartInfo	diskpartinfo;
	DiskCHSInfo	diskchsinfo;
	UB	ub[256];

	/* Υå */
	if (argc < 2) {
		printf("usage: diskinf [devicename]\n");
		goto fin0;
	}

	/* ǥХ򥪡ץ */
	err = opn_dev(argv[1], D_READ, NULL);
	if (err < ER_OK) {
		printf("opn_dev(): %d\n", err);
		goto fin0;
	}
	dd = err;

	/* DN_DISKEVENT */
	printf("[DN_DISKEVENT]\n");
	err = rea_dev(dd, DN_DISKEVENT, (UB *)&id, sizeof(id), &rsz, NULL);
	if (err < ER_OK) {
		printf("rea_dev(): %d\n", err);
	} else {
		printf("ID        : %d\n", id);
	}

	/* DN_DISKINFO */
	printf("[DN_DISKINFO]\n");
	err = rea_dev(dd, DN_DISKINFO, (UB *)&diskinfo, sizeof(diskinfo),
		      &rsz, NULL);
	if (err < ER_OK) {
		printf("rea_dev(): %d\n", err);
	} else {
		printf("format    : %d\n", diskinfo.format);
		printf("protect   : %s\n", diskinfo.protect ? "o" : "x");
		printf("removable : %s\n", diskinfo.removable ? "o" : "x");
		printf("rsv       : %d\n", diskinfo.rsv);
		printf("blocksize : %d\n", diskinfo.blocksize);
		printf("blockcont : %d\n", diskinfo.blockcont);
	}

	/* DN_DISKMEMADR */
	printf("[DN_DISKMEMADR]\n");
	err = rea_dev(dd, DN_DISKMEMADR, (UB *)&vp, sizeof(vp), &rsz, NULL);
	if (err < ER_OK) {
		printf("rea_dev(): %d\n", err);
	} else {
		printf("VP        : 0x%08x\n", (UW)vp);
	}
		       
	/* DN_DISKPARTINFO */
	printf("[DN_DISKPARTINFO]\n");
	err = rea_dev(dd, DN_DISKPARTINFO, (UB *)&diskpartinfo,
		      sizeof(diskpartinfo), &rsz, NULL);
	if (err < ER_OK) {
		printf("rea_dev(): %d\n", err);
	} else {
		printf("systemid  : 0x%02x\n", diskpartinfo.systemid);
		printf("startblock: %d\n", diskpartinfo.startblock);
		printf("endblock  : %d\n", diskpartinfo.endblock);
	}

	/* DN_DISKCHSINFO */
	printf("[DN_DISKCHSINFO]\n");
	err = rea_dev(dd, DN_DISKCHSINFO, (UB *)&diskchsinfo,
		      sizeof(diskchsinfo), &rsz, NULL);
	if (err < ER_OK) {
		printf("rea_dev(): %d\n", err);
	} else {
		printf("cylinder  : %d\n", diskchsinfo.cylinder);
		printf("head      : %d\n", diskchsinfo.head);
		printf("sector    : %d\n", diskchsinfo.sector);
	}

	/* DN_DISKIDINFO */
	printf("[DN_DISKIDINFO]\n");
	err = rea_dev(dd, DN_DISKIDINFO, ub, sizeof(ub), &rsz, NULL);
	if (err < ER_OK) {
		printf("rea_dev(): %d\n", err);
	} else {
		for (i = 0; i < rsz; i++) {
			printf("%02x ", ub[i]);
			if (!((i + 1) % 16)) printf("\n");
		}
	}

	err = ER_OK;
// fin1:
	cls_dev(dd, 0, NULL);
 fin0:
	return err;
}

#if 0


    WERR	err;
    W		dd, fd, imgsize, rsz;
    UB		*buf;
    LINK	link;
    DiskInfo	di;

    /* ǥХ򥪡ץ */
    err = opn_dev(target_drive[*argv[1] - L''], D_READ | D_EXCL, NULL);
    if(err < ER_OK) {
	printf("opn_dev(): %d\n", err);
	goto	fin0;
    }
    dd = err;

    /* ǥΥå */
    err = rea_dev(dd, DN_DISKINFO, (UB *)&di, sizeof(di), &rsz, NULL);
    if(err < ER_OK) {
	printf("rea_dev(): %d\n", err);
	goto	fin1;
    }
    if(di.format != DiskFmt_2DD && di.format != DiskFmt_2HD) {
	printf("unsupported media: %d\n", di.format);
	err = ER_DEV;
	goto fin1;
    }
    DP(("di.format: %d\n", di.format));

    /* ᡼η׻ */
    imgsize = di.blocksize * di.blockcont;
    DP(("di.blocksize: %d  di.blockcont: %d  imgsize: %d\n",
	di.blocksize, di.blockcont, imgsize));

    /* γ */
    buf = malloc(imgsize);
    if(buf == NULL) {
	printf("malloc(): NULL\n");
	err = ER_NOMEM;
	goto fin1;
    }

    /* ᡼ɤ߹ */
    err = rea_dev(dd, 0, buf, di.blockcont, &rsz, NULL);
    if(err < ER_OK) {
	printf("rea_dev(): %d\n", err);
	goto fin2;
    }

    /* ϥեؤΥ󥯼Ф */
    err = get_lnk(NULL, &link, F_NORM);
    if(err < 0) {
	printf("get_lnk(): %d\n", err);
	goto fin2;
    }

    /* ϥեκ */
    err = cre_fil(&link, argv[2], NULL, 0, F_FIX);
    if(err < 0) {
	printf("cre_fil(): %d\n", err);
	goto fin2;
    }
    fd = err;

    /* 쥳ɤν񤭹 */
    err = apd_rec(fd, buf, imgsize, 31, 0, 0);
    if(err < 0) {
	printf("apd_rec(): %d\n", err);
	goto fin3;
    }

    err = ER_OK;

 fin3:
    cls_fil(fd);
 fin2:
    free(buf);
 fin1:
    cls_dev(dd, 0, NULL);	/* dd, 1, NULL ȥǥȴܤˤʤ */
 fin0:
    return err;
}
#endif
